@@ -1,7 +1,15 @@
-use crate::broadword;
-use serde::{Deserialize, Serialize};
+pub mod unary;
 
-const WORD_LEN: usize = std::mem::size_of::<usize>() * 8;
+use std::io::{Read, Write};
+use std::mem::size_of;
+
+use anyhow::Result;
+use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
+
+use crate::bit_vector::unary::UnaryIterator;
+use crate::{broadword, util};
+
+pub(crate) const WORD_LEN: usize = std::mem::size_of::<usize>() * 8;
 
 /// Bit vector in a plain format, supporting some utilities such as update, chunking, and predecessor queries.
 ///
@@ -23,8 +31,15 @@ const WORD_LEN: usize = std::mem::size_of::<usize>() * 8;
 /// assert_eq!(bv.predecessor0(2), Some(2));
 /// assert_eq!(bv.successor1(1), Some(3));
 /// assert_eq!(bv.successor0(1), Some(1));
+///
+/// let mut bytes = vec![];
+/// let size = bv.serialize_into(&mut bytes).unwrap();
+/// let other = BitVector::deserialize_from(&bytes[..]).unwrap();
+/// assert_eq!(bv, other);
+/// assert_eq!(size, bytes.len());
+/// assert_eq!(size, bv.size_in_bytes());
 /// ```
-#[derive(Serialize, Deserialize, Default)]
+#[derive(Default, Clone, PartialEq, Eq)]
 pub struct BitVector {
     words: Vec<usize>,
     len: usize,
@@ -36,6 +51,14 @@ impl BitVector {
         Self::default()
     }
 
+    /// Creates a new [`BitVector`] of `len` bits.
+    pub fn with_len(len: usize) -> Self {
+        Self {
+            words: vec![0; Self::words_for(len)],
+            len,
+        }
+    }
+
     /// Creates a new [`BitVector`] that `capa` bits are reserved.
     pub fn with_capacity(capa: usize) -> Self {
         Self {
@@ -44,12 +67,32 @@ impl BitVector {
         }
     }
 
-    /// Creates a new [`BitVector`] of `len` bits.
-    pub fn with_len(len: usize) -> Self {
-        Self {
-            words: vec![0; Self::words_for(len)],
-            len,
-        }
+    /// Serializes the data structure into the writer,
+    /// returning the number of serialized bytes.
+    ///
+    /// # Arguments
+    ///
+    /// - `writer`: `std::io::Write` variable.
+    pub fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
+        let mem = util::int_vector::serialize_into(&self.words, &mut writer)?;
+        writer.write_u64::<LittleEndian>(self.len as u64)?;
+        Ok(mem + size_of::<u64>())
+    }
+
+    /// Deserializes the data structure from the reader.
+    ///
+    /// # Arguments
+    ///
+    /// - `reader`: `std::io::Read` variable.
+    pub fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
+        let words = util::int_vector::deserialize_from(&mut reader)?;
+        let len = reader.read_u64::<LittleEndian>()? as usize;
+        Ok(Self { words, len })
+    }
+
+    /// Returns the number of bytes to serialize the data structure.
+    pub fn size_in_bytes(&self) -> usize {
+        util::int_vector::size_in_bytes(&self.words) + size_of::<u64>()
     }
 
     /// Creates a new [`BitVector`] from input bitset `bits`.
@@ -272,8 +315,8 @@ impl BitVector {
         } else {
             let cur_word = self.words.last_mut().unwrap();
             *cur_word |= bits << pos_in_word;
-            if len > 64 - pos_in_word {
-                self.words.push(bits >> (64 - pos_in_word));
+            if len > WORD_LEN - pos_in_word {
+                self.words.push(bits >> (WORD_LEN - pos_in_word));
             }
         }
         self.len += len;
@@ -305,7 +348,7 @@ impl BitVector {
         let mut word = (self.words[block] << shift) >> shift;
         loop {
             if let Some(ret) = broadword::msb(word) {
-                return Some(block * 64 + ret);
+                return Some(block * WORD_LEN + ret);
             } else if block == 0 {
                 return None;
             }
@@ -340,7 +383,7 @@ impl BitVector {
         let mut word = (self.words[block] >> shift) << shift;
         loop {
             if let Some(ret) = broadword::lsb(word) {
-                return Some(block * 64 + ret).filter(|&i| i < self.len());
+                return Some(block * WORD_LEN + ret).filter(|&i| i < self.len());
             }
             block += 1;
             if block == self.words.len() {
@@ -376,7 +419,7 @@ impl BitVector {
         let mut word = (!self.words[block] << shift) >> shift;
         loop {
             if let Some(ret) = broadword::msb(word) {
-                return Some(block * 64 + ret);
+                return Some(block * WORD_LEN + ret);
             } else if block == 0 {
                 return None;
             }
@@ -411,7 +454,7 @@ impl BitVector {
         let mut word = (!self.words[block] >> shift) << shift;
         loop {
             if let Some(ret) = broadword::lsb(word) {
-                return Some(block * 64 + ret).filter(|&i| i < self.len());
+                return Some(block * WORD_LEN + ret).filter(|&i| i < self.len());
             }
             block += 1;
             if block == self.words.len() {
@@ -421,6 +464,24 @@ impl BitVector {
         }
     }
 
+    /// Creates an iterator for enumerating positions of set bits.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use sucds::BitVector;
+    ///
+    /// let bv = BitVector::from_bits(&[false, true, false, false, true]);
+    /// let mut it = bv.unary_iter(1);
+    ///
+    /// assert_eq!(it.next(), Some(1));
+    /// assert_eq!(it.next(), Some(4));
+    /// assert_eq!(it.next(), None);
+    /// ```
+    pub fn unary_iter(&self, pos: usize) -> UnaryIterator {
+        UnaryIterator::new(self, pos)
+    }
+
     /// Gets the `word_pos`-th word.
     #[inline(always)]
     pub fn get_word(&self, word_pos: usize) -> usize {
@@ -588,12 +649,12 @@ mod tests {
 
     #[test]
     fn test_serialize() {
+        let mut bytes = vec![];
         let bv = BitVector::from_bits(&gen_random_bits(10000, 42));
-        let bytes = bincode::serialize(&bv).unwrap();
-        let other: BitVector = bincode::deserialize(&bytes).unwrap();
-        assert_eq!(bv.len(), other.len());
-        for i in 0..bv.len() {
-            assert_eq!(bv.get_bit(i), other.get_bit(i));
-        }
+        let size = bv.serialize_into(&mut bytes).unwrap();
+        let other = BitVector::deserialize_from(&bytes[..]).unwrap();
+        assert_eq!(bv, other);
+        assert_eq!(size, bytes.len());
+        assert_eq!(size, bv.size_in_bytes());
     }
 }
