@@ -1,3 +1,4 @@
+#![cfg(target_pointer_width = "64")]
 //! # `sucds`: Succinct data structures in Rust
 //!
 //! `sucds` contains some [succinct data structures](https://en.wikipedia.org/wiki/Succinct_data_structure) written in Rust.
@@ -10,26 +11,17 @@
 //! - [`BitVector`]
 //!   - Bit vector in a plain format, supporting some utilities such as update, chunking, and predecessor queries.
 //! - [`CompactVector`]
-//!   - Compact vector in which each integer is represented in the specified number of bits.
+//!   - Compact vector in which each integer is represented in the fixed number of bits.
 //! - [`RsBitVector`]
-//!   - Rank/select data structure over bit vector through Vigna's rank9 and hinted selection techniques.
+//!   - Rank/select data structure over bit vectors with Vigna's rank9 and hinted selection techniques.
 //! - [`DArray`]
-//!   - Constant-time select data structure over integer set through dense array technique by Okanohara and Sadakane.
+//!   - Constant-time select data structure over integer sets with the dense array technique by Okanohara and Sadakane.
 //! - [`EliasFano`]
-//!   - Compressed monotone sequence through Elias-Fano encoding.
+//!   - Compressed monotone sequence with Elias-Fano encoding.
 //! - [`EliasFanoList`]
-//!   - Compressed integer list through Elias-Fano gap encoding.
-//!
-//! ## Usage
-//!
-//! To use `sucds`, depend on it in your Cargo manifest:
-//!
-//! ```toml
-//! # Cargo.toml
-//!
-//! [dependencies]
-//! sucds = "0.1"
-//! ```
+//!   - Compressed integer list with prefix-summed Elias-Fano encoding.
+//! - [`WaveletMatrix`]
+//!   - Space-efficient data structure providing myriad operations over integer sequences.
 //!
 //! ## Limitation
 //!
@@ -43,6 +35,7 @@ pub mod elias_fano_list;
 pub mod intrinsics;
 pub mod rs_bit_vector;
 pub mod util;
+pub mod wavelet_matrix;
 
 pub use bit_vector::BitVector;
 pub use compact_vector::CompactVector;
@@ -51,3 +44,5 @@ pub use elias_fano::EliasFano;
 pub use elias_fano::EliasFanoBuilder;
 pub use elias_fano_list::EliasFanoList;
 pub use rs_bit_vector::RsBitVector;
+pub use wavelet_matrix::WaveletMatrix;
+pub use wavelet_matrix::WaveletMatrixBuilder;
