@@ -1,13 +1,18 @@
 #![cfg(target_pointer_width = "64")]
 
-use crate::{broadword, BitVector};
-use serde::{Deserialize, Serialize};
+use std::io::{Read, Write};
+use std::mem::size_of;
+
+use anyhow::Result;
+use byteorder::{ReadBytesExt, WriteBytesExt};
+
+use crate::{broadword, util, BitVector};
 
 const BLOCK_LEN: usize = 8;
 const SELECT_ONES_PER_HINT: usize = 64 * BLOCK_LEN * 2;
 const SELECT_ZEROS_PER_HINT: usize = SELECT_ONES_PER_HINT;
 
-/// Rank/select data structure over a bit vector through Vigna's rank9 and hinted selection techniques.
+/// Rank/select data structure over bit vectors with Vigna's rank9 and hinted selection techniques.
 ///
 /// [`RsBitVector`] builds rank/select indexes on a bit vector.
 /// For a bit vector of $`n`$ bits,
@@ -22,17 +27,25 @@ const SELECT_ZEROS_PER_HINT: usize = SELECT_ONES_PER_HINT;
 /// use sucds::RsBitVector;
 ///
 /// let bv = RsBitVector::from_bits(&[true, false, false, true], true, true);
+///
 /// assert_eq!(bv.get_bit(1), false);
 /// assert_eq!(bv.rank1(1), 1);
 /// assert_eq!(bv.rank0(1), 0);
 /// assert_eq!(bv.select1(1), 3);
 /// assert_eq!(bv.select0(0), 1);
+///
+/// let mut bytes = vec![];
+/// let size = bv.serialize_into(&mut bytes).unwrap();
+/// let other = RsBitVector::deserialize_from(&bytes[..]).unwrap();
+/// assert_eq!(bv, other);
+/// assert_eq!(size, bytes.len());
+/// assert_eq!(size, bv.size_in_bytes());
 /// ```
 ///
 /// # References
 ///
 ///  - S. Vigna, "Broadword implementation of rank/select queries," In WEA, 2008.
-#[derive(Serialize, Deserialize, Default, Debug)]
+#[derive(Default, Debug, Clone, PartialEq, Eq)]
 pub struct RsBitVector {
     bv: BitVector,
     block_rank_pairs: Vec<usize>,
@@ -99,6 +112,70 @@ impl RsBitVector {
         Self::new(BitVector::from_bits(bits), select1_hints, select0_hints)
     }
 
+    /// Serializes the data structure into the writer,
+    /// returning the number of serialized bytes.
+    ///
+    /// # Arguments
+    ///
+    /// - `writer`: `std::io::Write` variable.
+    pub fn serialize_into<W: Write>(&self, mut writer: W) -> Result<usize> {
+        let mut mem = self.bv.serialize_into(&mut writer)?;
+        mem += util::int_vector::serialize_into(&self.block_rank_pairs, &mut writer)?;
+        if let Some(select1_hints) = &self.select1_hints {
+            writer.write_u8(1)?;
+            mem += util::int_vector::serialize_into(select1_hints, &mut writer)?;
+        } else {
+            writer.write_u8(0)?;
+        }
+        if let Some(select0_hints) = &self.select0_hints {
+            writer.write_u8(1)?;
+            mem += util::int_vector::serialize_into(select0_hints, &mut writer)?;
+        } else {
+            writer.write_u8(0)?;
+        }
+        Ok(mem + size_of::<u8>() * 2)
+    }
+
+    /// Deserializes the data structure from the reader.
+    ///
+    /// # Arguments
+    ///
+    /// - `reader`: `std::io::Read` variable.
+    pub fn deserialize_from<R: Read>(mut reader: R) -> Result<Self> {
+        let bv = BitVector::deserialize_from(&mut reader)?;
+        let block_rank_pairs = util::int_vector::deserialize_from(&mut reader)?;
+        let select1_hints = if reader.read_u8()? != 0 {
+            Some(util::int_vector::deserialize_from(&mut reader)?)
+        } else {
+            None
+        };
+        let select0_hints = if reader.read_u8()? != 0 {
+            Some(util::int_vector::deserialize_from(&mut reader)?)
+        } else {
+            None
+        };
+        Ok(Self {
+            bv,
+            block_rank_pairs,
+            select1_hints,
+            select0_hints,
+        })
+    }
+
+    /// Returns the number of bytes to serialize the data structure.
+    pub fn size_in_bytes(&self) -> usize {
+        self.bv.size_in_bytes()
+            + util::int_vector::size_in_bytes(&self.block_rank_pairs)
+            + size_of::<u8>()
+            + self.select1_hints.as_ref().map_or(0, |select1_hints| {
+                util::int_vector::size_in_bytes(select1_hints)
+            })
+            + size_of::<u8>()
+            + self.select0_hints.as_ref().map_or(0, |select0_hints| {
+                util::int_vector::size_in_bytes(select0_hints)
+            })
+    }
+
     /// Gets the `pos`-th bit.
     ///
     /// # Arguments
@@ -483,14 +560,12 @@ mod tests {
 
     #[test]
     fn test_serialize() {
-        let bits = gen_random_bits(10000, 42);
-        let bv = RsBitVector::from_bits(&bits, true, true);
-        let bytes = bincode::serialize(&bv).unwrap();
-        let other: RsBitVector = bincode::deserialize(&bytes).unwrap();
-        assert_eq!(bv.len(), other.len());
-        assert_eq!(bv.num_ones(), other.num_ones());
-        assert_eq!(bv.num_zeros(), other.num_zeros());
-        test_rank_select1(&bits, &other);
-        test_rank_select0(&bits, &other);
+        let mut bytes = vec![];
+        let bv = RsBitVector::from_bits(&gen_random_bits(10000, 42), true, true);
+        let size = bv.serialize_into(&mut bytes).unwrap();
+        let other = RsBitVector::deserialize_from(&bytes[..]).unwrap();
+        assert_eq!(bv, other);
+        assert_eq!(size, bytes.len());
+        assert_eq!(size, bv.size_in_bytes());
     }
 }
